/* Keypad class, by Lars Skovlund */
(script 901)

(include "sci.sh")
(include "keypad.sh")

(use "obj")
(use "feature")
(use "main")

(class Keypad of EventHandler
	(properties
		outputString	0
		maxLength	0
		length		0
		autoExit	FALSE
		backView	0
		backPri		-2
		btnSound	0
		longestText	1
		script		0
		button		0
		state		FALSE
		special		0 // Work around property chicken-and-egg thing.
		requireMax	FALSE
	)

	(method (init buttons)
		(self:
			add(rest buttons)
			eachElementDo(#init self (+ backPri 1))
		)
		(if ((>= backPri 0) and IsObject(backView))
			(send backView:setPri(backPri))
		)
		(if (IsObject(backView))
			(send backView:init())
		)
		= length 0
		StrCpy(outputString "")
		(send script:changeState(KEYPAD_INIT_STATE))
	)
	(method (show)
		(if (IsObject(backView))
			(send backView:show())
		)
		(self:eachElementDo(#show))
		(ProgramControl())
		= state TRUE
		(send (send gRoom:controls()):add(self))
	)

	(method (hide doCue)
		(if (IsObject(backView))
			(send backView:hide())
		)
		(self:eachElementDo(#hide))
		(PlayerControl())
		(send (send gRoom:controls()):delete(self))
		(if (IsObject(script) and (((>= paramTotal 1) and doCue) or 
				(== paramTotal 0)))
			(send script:cue())
		)
		Animate((send gCast:elements()) FALSE)
		= state FALSE
	)

	(method (backspace)
		(if (>= length longestText)
			(StrAt(outputString (- StrLen(outputString) longestText) 0))
			(= length (- length longestText))
		)
	)

	(method (draw)) /* Satisfy the object hierarchy */
	(method (setSize))

	(method (depress which)
		(var howMuch, appender)
		(switch (send which:special())
			(case KEYPAD_SPECIAL_NONE
				= howMuch (- maxLength length)
				(if (> howMuch longestText) (= howMuch longestText))
				(if (> howMuch 0)
					// Argh, cannot use StrCat; it does not support a length argument
					= appender (StrEnd(outputString))
					StrCpy(appender (send which:text()) (+ howMuch 1))
					+= length howMuch
					(if (autoExit and (>= length maxLength))
						(self:hide(TRUE))
					)
				)
			)
			(case KEYPAD_SPECIAL_BKSP
				(self:backspace())
			)
			(case KEYPAD_SPECIAL_ENTER
				(if ((== length maxLength) or (not requireMax))
					(self:hide(TRUE))) (else
					(if (script) (send script:changeState(KEYPAD_ERROR_STATE))))
			)
			(case KEYPAD_SPECIAL_CANCEL
				(self:hide(FALSE))
			)
		)
	)

/* Copied from the template game */
	(method (setScript newScript params)
		(if(script)
			(send script:dispose())
		)
		(if(newScript)
			(send newScript:init(self rest params))
		)
	)
)

(class KeypadButton of Prop
	(properties
		client		0
		special		0
		text		0
	)

	(method (init theClient priority)
		(super:init())
		(self:hide())
		(if (>= priority 0)
			(self:setPri(priority))
		)
		= client theClient
	)

	(method (handleEvent event)
		(var theX, theY, btn)

		(if ((== (send event:type()) evMOUSEBUTTON) and(== (cel 1)))
			(return))
		(if (== (send event:type()) evMOUSERELEASE)
			(self:setCel(0))
		)

		= theX (send event:x())
		= theY (send event:y())
		(if ((< theX nsLeft) or
		    (>= theX nsRight) or
		    (< theY nsTop) or
		    (>= theY nsBottom))
			(return)
		)

		(if (== (send event:type()) evMOUSEBUTTON)
			(self:setCel(1))
			(if (send client:btnSound())
				(send (send client:btnSound()):play())
			)
			(send client:button(self))
		)

		= btn (send client:button)
		(if ((== (send event:type()) evMOUSERELEASE) and
		     (== btn self))
			(send client:depress(btn) button(0))
		)
	)
)


(class DisplayKeypad of Keypad
	(properties
		underBits	0
		x		0
		y		0
		font		0
		color		0
		back		0
		width		192
		align		0
	)

	(method (depress which)
		(super:depress(which))
		(if (underBits) 
			(Display("" dsRESTOREPIXELS underBits))
	 		 = underBits 0
			)
		(if (state)
			(= underBits Display(outputString dsCOORD x y dsCOLOR color dsBACKGROUND back dsFONT font dsALIGN align dsWIDTH width dsSAVEPIXELS))
		)
	)

	(method (hide params)
		(super:hide(rest params))
		(if (underBits) 
			(Display("" dsRESTOREPIXELS underBits))
	 		 = underBits 0
			)
	)
)